﻿using System;
using System.IO;
using System.IO.Pipes;
using System.Threading;
using System.Threading.Tasks;

namespace ServerSIP
{
    class cVaxPipeReadData
    {
        public Byte[] m_aDataRead = null;
        public cVaxPipeClient m_objPipeClient = null;
    }

    /////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////

    public interface iVaxPipeClient
    {
        void OnVaxPipeClientRecvPCM(cVaxPipeClient objPipeClient, Byte[] aDataPCM, int nSizePCM);
        void OnVaxPipeClientDisconnected(cVaxPipeClient objPipeClient);
    }

    /////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////
    public class cVaxPipeClient
    {
        NamedPipeClientStream m_objHandlePipe = null;
        iVaxPipeClient m_iPipeClient;

        public cVaxPipeClient(iVaxPipeClient iPipeClient)
        {
            m_iPipeClient = iPipeClient;
            m_objHandlePipe = null;
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////////////
        public async Task<Boolean> CreatePipeAsyncReadWrite(String sPipeName)
        {
            m_objHandlePipe = new NamedPipeClientStream(".",
                             sPipeName,
                             PipeAccessRights.ReadWrite | PipeAccessRights.WriteAttributes,
                             PipeOptions.Asynchronous,
                             System.Security.Principal.TokenImpersonationLevel.None,
                             System.IO.HandleInheritability.None);

            return await ConnectPipeAsync();
        }
        public async Task<Boolean> CreatePipeAsyncReadOnly(String sPipeName)
        {
            m_objHandlePipe = new NamedPipeClientStream(".",
                             sPipeName,
                             PipeAccessRights.ReadData | PipeAccessRights.WriteAttributes,
                             PipeOptions.Asynchronous,
                             System.Security.Principal.TokenImpersonationLevel.None,
                             System.IO.HandleInheritability.None);

            return await ConnectPipeAsync();
        }

        private async Task<Boolean> ConnectPipeAsync()
        {
            try
            {
                await m_objHandlePipe.ConnectAsync(2000);

                if (m_objHandlePipe.IsConnected == false)
                {
                    ClosePipe();
                    return false;
                }

                m_objHandlePipe.ReadMode = PipeTransmissionMode.Message;
            }
            catch (TimeoutException ex)
            {
                Console.WriteLine("Connection attempt timed out: " + ex.Message);

                ClosePipe();
                return false;
            }
            catch (IOException ex)
            {
                Console.WriteLine("I/O error occurred: " + ex.Message);

                ClosePipe();
                return false;
            }
            catch (Exception ex)
            {
                Console.WriteLine("An unexpected error occurred: " + ex.Message);

                ClosePipe();
                return false;
            }

            BeginReadPipe();
            return true;
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////////////

        static void WriteCallback(IAsyncResult asyncResult)
        {

        }
        static void ReadCallback(IAsyncResult asyncResult)
        {
            cVaxPipeReadData objReadData = asyncResult.AsyncState as cVaxPipeReadData;
            objReadData.m_objPipeClient.OnReadPipeCompleted(asyncResult, objReadData);
        }

        private void OnReadPipeCompleted(IAsyncResult asyncResult, cVaxPipeReadData objReadData)
        {
            int nReadSize = m_objHandlePipe.EndRead(asyncResult);

            if (nReadSize <= 0)
            {
                m_iPipeClient.OnVaxPipeClientDisconnected(this);
                return;
            }

            m_iPipeClient.OnVaxPipeClientRecvPCM(this, objReadData.m_aDataRead, nReadSize);

            BeginReadPipe();
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////////////

        private Boolean BeginReadPipe()
        {
            if (m_objHandlePipe == null)
                return false;

            cVaxPipeReadData objReadData = new cVaxPipeReadData();

            objReadData.m_aDataRead = new byte[6400];
            objReadData.m_objPipeClient = this;

            IAsyncResult hResult = m_objHandlePipe.BeginRead(objReadData.m_aDataRead, 0, objReadData.m_aDataRead.Length, new AsyncCallback(ReadCallback), objReadData);

            return true;
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////////////
        public Boolean WritePipe(Byte[] aDataPCM, int nSizePCM)
        {
            if (m_objHandlePipe == null)
                return false;

            Byte[] aDataWrite = new Byte[nSizePCM];
            Array.Copy(aDataPCM, aDataWrite, nSizePCM);

            IAsyncResult hResult = m_objHandlePipe.BeginWrite(aDataWrite, 0, nSizePCM, new AsyncCallback(WriteCallback), null);

            return true;
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////////////

        public Boolean IsPipeConnected()
        {
            if (m_objHandlePipe == null)
                return false;

            Boolean bConnected = m_objHandlePipe.IsConnected;
            return bConnected;
        }

        public void ClosePipe()
        {
            if (m_objHandlePipe == null)
                return;

            m_objHandlePipe.Close();
            m_objHandlePipe = null;
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////////////
        /*
        public Boolean CreatePipe(String sPipeName)
        {
            m_objHandlePipe = new NamedPipeClientStream(".",
                             sPipeName,
                             PipeAccessRights.ReadData | PipeAccessRights.WriteAttributes,
                             PipeOptions.Asynchronous,
                             System.Security.Principal.TokenImpersonationLevel.None,
                             System.IO.HandleInheritability.None);
            try
            {
                m_objHandlePipe.Connect(3000); // 3000 ms timeout
                m_objHandlePipe.ReadMode = PipeTransmissionMode.Message;
            }
            catch (TimeoutException ex)
            {
                Console.WriteLine("Connection attempt timed out: " + ex.Message);

                ClosePipe();
                return false;
            }
            catch (IOException ex)
            {
                Console.WriteLine("I/O error occurred: " + ex.Message);

                ClosePipe();
                return false;
            }
            catch (Exception ex)
            {
                Console.WriteLine("An unexpected error occurred: " + ex.Message);

                ClosePipe();
                return false;
            }

            return true;
        }

        public Boolean ReadPipe(Byte[] aReadBuff, int nSizeBuff, ref int nSizeRead)
        {
            if (m_objHandlePipe == null)
                return false;

            int nRead = m_objHandlePipe.Read(aReadBuff, 0, nSizeBuff);
            if (nRead == 0) return false;

            nSizeRead = nRead;
            return true;
        }

        */
    }
}
