using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using ServerSIP;

namespace ServerSIP
{
    public partial class LoginForm : Form
    {
        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////
                       
        public LoginForm()
        {
            InitializeComponent();
        }
               
        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void BtnAddLogin_Click(object sender, EventArgs e)
        {
            if(AddLoginChecks() == false)
                return;

            if(ComboLogin.FindStringExact(ComboLogin.Text) != -1)
            {
                if(MessageBox.Show("Login already exists, do you want to continue?", "VaxVoIP", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.No)
                {
                    return;
                }
            }

            ////////////////////////////////////////////////////////////////////////////////////////////

            String sUserPrefix = "", sDialPrefix = "";
            String sToPeerName = "";

            GetRouteUI(ref sUserPrefix, ref sDialPrefix, ref sToPeerName);

            ////////////////////////////////////////////////////////////////////////////////////////////

            String sCodec = GetAudioCodecUI();
            int nVoiceTypeId = GetUserTypeUI();

            ////////////////////////////////////////////////////////////////////////////////////////////

            if (!cVaxUser.AddUser(ComboLogin.Text, EditPassword.Text, sCodec, CheckBoxBlockAccount.Checked, nVoiceTypeId, sUserPrefix, sDialPrefix, sToPeerName))
            {
                MessageBox.Show("Login could not be created or Please start the server first.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            ////////////////////////////////////////////////////////////////////////////////////////////

            if(!ComboLogin.Items.Contains(ComboLogin.Text))
                ComboLogin.Items.Add(ComboLogin.Text);

            MessageBox.Show("Login added/updated successfully.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void BtnRemoveLogin_Click(object sender, EventArgs e)
        {
            if (ComboLogin.Text.Length == 0)
            {
                MessageBox.Show("Please enter a login name.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            int nIndex = ComboLogin.FindStringExact(ComboLogin.Text);
            if (nIndex == -1) return;

            cVaxUser.RemoveUser(ComboLogin.Text);
            ComboLogin.Items.RemoveAt(nIndex);

            ClearDlgUI();

            if (ComboLogin.Items.Count > 0)
                ComboLogin.SelectedIndex = 0;

            ShowLoginDetails();

            MessageBox.Show("Login removed successfully!", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }
                
        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void ComboLogin_SelectedIndexChanged(object sender, EventArgs e)
        {
            ShowLoginDetails();
        }

        private void BtnClearLogin_Click(object sender, EventArgs e)
        {
            ClearDlgUI();
        }
                
        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void LoadLoginAll()
        {
            ComboLogin.Items.Clear();

            ArrayList objListUsers = new ArrayList();
            cStoreUser.GetUserAll(ref objListUsers);

            if(objListUsers.Count <= 0) return;

            for (int nIndex = 0; nIndex < objListUsers.Count; nIndex++)
                ComboLogin.Items.Add(objListUsers[nIndex]);

            if(ComboLogin.Items.Count > 0)
                ComboLogin.SelectedIndex = 0;
        }

        private void LoadRouteLineAll()
        {
            ArrayList objListLines = new ArrayList();

            cStoreLine.GetLineNameAll(ref objListLines);
            if(objListLines.Count <= 0) return;

            for (int nIndex = 0; nIndex < objListLines.Count; nIndex++)
            {
                String sToPeerName = objListLines[nIndex].ToString();
                ComboLoginRoute.Items.Add(sToPeerName);
            }
                        
            if (ComboLoginRoute.Items.Count > 0)
                ComboLoginRoute.SelectedIndex = 0;
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////
   
        public void OnSetActive()
        {
            ComboLoginRoute.Items.Clear();

            LoadLoginAll();
            LoadRouteLineAll();

            ShowLoginDetails();
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void ShowLoginDetails()
        {
            String sLogin = ComboLogin.Text;
            
            if(sLogin.Length == 0)
                return;
            
            String sPassword = "", sUserPrefix = "";
            String sDialPrefix = "", sAudioCodecList = "", sToPeerName = "";
            
            Boolean bBlocked = true;
            int nVoiceTypeId = cVaxServerCOM.VAX_AUDIO_TYPE_NORMAL;

            cStoreUser.GetUser(sLogin, ref sPassword, ref sAudioCodecList, ref bBlocked, ref nVoiceTypeId, ref sUserPrefix, ref sDialPrefix, ref sToPeerName);
            UpdateDlgUI(sLogin, sPassword, sAudioCodecList, bBlocked, nVoiceTypeId, sUserPrefix, sDialPrefix, sToPeerName);
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void UpdateDlgUI(String sLogin, String sPassword, String sAudioCodecList, Boolean bBlocked, int nVoiceTypeId, String sUserPrefix, String sDialPrefix, String sToPeerName)
        {
            if(sLogin.Length == 0)
                return;

            EditPassword.Text = sPassword;
            CheckBoxBlockAccount.Checked = bBlocked;

            UpdateUserTypeUI(nVoiceTypeId);
            UpdateAudioCodecUI(sAudioCodecList);

            UpdateRouteUI(sUserPrefix, sDialPrefix, sToPeerName);
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void UpdateAudioCodecUI(String sAudioCodecList)
        {
            ListBoxEnableCodec.Items.Clear();
            ListBoxDisableCodec.Items.Clear();

            for (int nCount = 0; nCount < cVaxServerCOM.VAX_CODEC_TOTAL_COUNT; nCount++)
                ListBoxDisableCodec.Items.Add(cCodec.GetCodecName(nCount));

            String sCodecName = "";

            for (int nCount = 0; nCount < sAudioCodecList.Length; nCount++)
            {
                sCodecName = cCodec.GetCodecName(sAudioCodecList[nCount]);

                ListBoxEnableCodec.Items.Add(sCodecName);
                ListBoxDisableCodec.Items.Remove(sCodecName);
            }
        }
                
        private String GetAudioCodecUI()
        {
            String sCodec = "";

            for (int nCount = 0; nCount < ListBoxEnableCodec.Items.Count; nCount++)
                sCodec += cCodec.GetCodecNo((String)ListBoxEnableCodec.Items[nCount]);

            return sCodec;
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void UpdateRouteUI(String sUserPrefix, String sDialPrefix, String sToPeerName)
        {
            EditDialPrefix.Text = sDialPrefix;
            EditCallPrefix.Text = sUserPrefix;
                        
            int nIndex = ComboLoginRoute.FindStringExact(sToPeerName);

            if (nIndex != -1)
            {
                ComboLoginRoute.SelectedIndex = nIndex;
            }
            else
            {
                if (ComboLoginRoute.Items.Count > 0)
                    ComboLoginRoute.SelectedIndex = 0;
            }
        }

        private void GetRouteUI(ref String sUserPrefix, ref String sDialPrefix, ref String sToPeerName)
        {
            int nSelectedIndex = ComboLoginRoute.SelectedIndex;

            sUserPrefix = ""; sDialPrefix = "";
            sToPeerName = "";

            if (nSelectedIndex == -1 || EditCallPrefix.Text.Length == 0)
                return;

            sUserPrefix = EditCallPrefix.Text;
            sDialPrefix = EditDialPrefix.Text;

            sToPeerName = ComboLoginRoute.Text;
        }
        
        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void UpdateUserTypeUI(int nUserType)
        {
            if (nUserType == cVaxServerCOM.VAX_AUDIO_TYPE_ADMIN)
                RadioTypeAdmin.Checked = true;

            else if (nUserType == cVaxServerCOM.VAX_AUDIO_TYPE_USER)
                RadioTypeAgent.Checked = true;

            else if (nUserType == cVaxServerCOM.VAX_AUDIO_TYPE_NORMAL)
                RadioTypeNormal.Checked = true;
            else
                RadioTypeNormal.Checked = true;
        }

        private int GetUserTypeUI()
        {
            if (RadioTypeAdmin.Checked == true)
            {
                return cVaxServerCOM.VAX_AUDIO_TYPE_ADMIN;
            }

            if (RadioTypeAgent.Checked == true)
            {
                return cVaxServerCOM.VAX_AUDIO_TYPE_USER;
            }

            if (RadioTypeNormal.Checked == true)
            {
                return cVaxServerCOM.VAX_AUDIO_TYPE_NORMAL;
            }

            return cVaxServerCOM.VAX_AUDIO_TYPE_NORMAL;
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void ClearDlgUI()
        {
            ComboLogin.Text = "";
            EditPassword.Text = "";
            
            CheckBoxBlockAccount.Checked = false;
            RadioTypeNormal.Checked = true;

            ListBoxEnableCodec.Items.Clear();
            ListBoxDisableCodec.Items.Clear();

            ListBoxEnableCodec.Items.Add(cCodec.AUDIO_CODEC_NAME_G711U);
            ListBoxEnableCodec.Items.Add(cCodec.AUDIO_CODEC_NAME_G711A);
            ListBoxEnableCodec.Items.Add(cCodec.AUDIO_CODEC_NAME_ILBC);
            ListBoxEnableCodec.Items.Add(cCodec.AUDIO_CODEC_NAME_GSM610);
            ListBoxEnableCodec.Items.Add(cCodec.AUDIO_CODEC_NAME_G729);

            EditCallPrefix.Text = "";
            EditDialPrefix.Text = "";

            if (ComboLoginRoute.Items.Count > 0)
                ComboLoginRoute.SelectedIndex = 0;
        }
                
        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private Boolean AddLoginChecks()
        {
            if(ComboLogin.Text.Length == 0)
            {
                MessageBox.Show("Please enter login.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }
            
            if (!cVaxDefine.IsDigitAll(ComboLogin.Text))
            {
                MessageBox.Show("Login should contain only numerals 0 through 9. For example: 12, 324323, 800122, 2109756 etc.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }
            
            if(EditPassword.Text.Length == 0)
            {
                MessageBox.Show("Please enter login password.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            if(ComboLogin.Text.Length > 20 || EditPassword.Text.Length > 20)
            {
                MessageBox.Show("Login && password size should not be more than 20.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            if(ComboLogin.Text.Substring(0, 2) == "00")
            {
                MessageBox.Show("Please do not enter 00 as login prefix. 00 prefix is use to prepare dial plan.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            if(ListBoxEnableCodec.Items.Count == 0)
            {
                MessageBox.Show("Please select a codec.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            return true;
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void BtnDisableCodec_Click(object sender, EventArgs e)
        {
            if (ListBoxEnableCodec.SelectedItems.Count == 0) return;

            ListBoxDisableCodec.Items.Add(ListBoxEnableCodec.SelectedItem.ToString());
            ListBoxEnableCodec.Items.Remove(ListBoxEnableCodec.SelectedItem);
        }

        private void BtnEnableCodec_Click(object sender, EventArgs e)
        {
            if (ListBoxDisableCodec.SelectedItems.Count == 0) return;

            ListBoxEnableCodec.Items.Add(ListBoxDisableCodec.SelectedItem.ToString());
            ListBoxDisableCodec.Items.Remove(ListBoxDisableCodec.SelectedItem);
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////
    }
}