﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using ServerSIP;
using System.Collections;

namespace ServerSIP
{
    public partial class RoomForm : Form
    {
        public RoomForm()
        {
            InitializeComponent();
        }

        private void OnAddConfRoom(object sender, EventArgs e)
        {
            if(!AddRoomChecks())
                return;

            if(ComboRoomName.FindStringExact(ComboRoomName.Text) != -1)
            {
                if(MessageBox.Show("Room already exists, do you want to continue?", "VaxVoIP", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.No)
                {
                    return;
                }
            }

            if (!cVaxRoom.AddRoom(ComboRoomName.Text.Trim(), EditRoomNo.Text.Trim()))
            {
                MessageBox.Show("Room could not be added/updated. Please start the server first or can't assign same Dial-in No to multiple rooms.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if(!ComboRoomName.Items.Contains(ComboRoomName.Text))
                ComboRoomName.Items.Add(ComboRoomName.Text);

            MessageBox.Show("Room added/updated successfully.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        private void OnRemoveConfRoom(object sender, EventArgs e)
        {
            cVaxRoom.RemoveRoom(ComboRoomName.Text.Trim());

            int nIndex = ComboRoomName.FindStringExact(ComboRoomName.Text);
            if(nIndex == -1) return;

            cVaxRoom.RemoveRoom(ComboRoomName.Text);
            ComboRoomName.Items.RemoveAt(nIndex);

            ClearDlgUI();

            if (ComboRoomName.Items.Count > 0)
                ComboRoomName.SelectedIndex = 0;

            ShowRoomDetails();

            MessageBox.Show("Room removed successfully!", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        private Boolean AddRoomChecks()
        {
            if(ComboRoomName.Text.Length == 0)
            {
                MessageBox.Show("Please enter Room Name.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            if(EditRoomNo.Text.Length == 0)
            {
                MessageBox.Show("Room No can't be empty.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            if (!cVaxDefine.IsDigitAll(EditRoomNo.Text))
            {
                MessageBox.Show("Room No should contain only numerals 0 through 9. For example: 12, 324323, 800122, 2109756 etc.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }
                        
            return true;
        }

        private void ComboRoomName_SelectedIndexChanged(object sender, EventArgs e)
        {
            ShowRoomDetails();
        }

        private void ClearDlgUI()
        {
            EditRoomNo.Text = "";
        }

        private void ShowRoomDetails()
        {
            String sRoomName = ComboRoomName.Text;
            
            if (sRoomName.Length == 0)
                return;
            
            String sRoomNo = "";
            cStoreRoom.GetRoom(sRoomName, ref sRoomNo);

            EditRoomNo.Text = sRoomNo;
        }

        private void AddRooms()
        {
            ComboRoomName.Items.Clear();

            ArrayList objList = new ArrayList();
            cStoreRoom.GetRoomNameAll(ref objList);

            if (objList.Count <= 0) return;

            for (int nCount = 0; nCount < objList.Count; nCount++)
                ComboRoomName.Items.Add(objList[nCount]);

            if(ComboRoomName.Items.Count > 0)
                ComboRoomName.SelectedIndex = 0;
        }

        public void OnSetActive()
        {
            AddRooms();
            ShowRoomDetails();
        }
    }
}
