﻿using System;

namespace ServerSIP
{
    class cVaxCallSession
    {
        static cVaxCallSessionRoute m_objRouteCallSIP = null;

        cVaxAccessAudioPCM m_objAccessPCM;
        cVaxDetectAudioVAD m_objDetectVAD;

        cVaxCall m_objVaxCall;
        cVaxTransfer m_objVaxTransfer;
                        
        cVaxDTMF m_objVaxDTMF;
        cVaxDetectAMD m_objVaxAMD;
        cVaxRoom m_objVaxRoom;
                                
        public cVaxCallSession()
        {
            if (cVaxCallSession.m_objRouteCallSIP == null)
                cVaxCallSession.m_objRouteCallSIP = new cVaxCallSessionRoute();

            m_objAccessPCM = new cVaxAccessAudioPCM();
            m_objDetectVAD = new cVaxDetectAudioVAD();

            m_objVaxCall = new cVaxCall();
            m_objVaxTransfer = new cVaxTransfer();
                       
            m_objVaxDTMF = new cVaxDTMF();
            m_objVaxAMD = new cVaxDetectAMD();

            m_objVaxRoom = new cVaxRoom();
        }

        ////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////
        public void OnCallSessionClosed(int nSessionId, int nChannelId, int nReasonCode)
        {
            m_objAccessPCM.OnCallSessionClosed(nSessionId);
            m_objDetectVAD.OnCallSessionClosed(nSessionId);
        }

        ////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////

        public static void AddUserRoute(String sUserName, String sUserPrefix, String sDialPrefix, String sToPeerName)
        {
            cVaxCallSession.m_objRouteCallSIP.AddUserRoute(sUserName, sUserPrefix, sDialPrefix, sToPeerName);
        }

        public static void RemoveUserRoute(String sUserName)
        {
            cVaxCallSession.m_objRouteCallSIP.RemoveUserRoute(sUserName);
        }

        ///////////////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////////////

        public static void AddLineRoute(String sLineName, String sToPeerName, Boolean bRejected, String sRoomName)
        {
            cVaxCallSession.m_objRouteCallSIP.AddLineRoute(sLineName, sToPeerName, bRejected, sRoomName);
        }

        public static void RemoveLineRoute(String sLineName)
        {
            cVaxCallSession.m_objRouteCallSIP.RemoveLineRoute(sLineName);
        }

        ///////////////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////////////

        public static void ResetAll()
        {
            cVaxCallSession.m_objRouteCallSIP = new cVaxCallSessionRoute();
        }

        ///////////////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////////////
       
        private void GetFromPeerRouteInfo(int nFromPeerType, String sFromPeerName, String sDialNo, ref String sDialToNo, ref String sToPeerName, ref Boolean bRejected, ref String sRoomName)
        {
            if (nFromPeerType == cVaxServerCOM.VAX_PEER_TYPE_USER)
            {
                cVaxCallSession.m_objRouteCallSIP.GetUserRouteInfo(sFromPeerName, sDialNo, ref sDialToNo, ref sToPeerName);
                return;
            }

            cVaxCallSession.m_objRouteCallSIP.GetLineRouteInfo(sFromPeerName, sDialNo, ref sDialToNo, ref sToPeerName, ref bRejected, ref sRoomName);
        }

        ///////////////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////////////

        public void OnIncomingCall(int nSessionId, String sCallerName, String sCallerId, String sDialNo, int nFromPeerType, String sFromPeerName, String sUserAgentName, String sFromIP, int nFromPort)
        {
            String sToPeerName = "", sDialToNo = "";
            String sRoomName = "";
                        
            Boolean bRejected = false;

            GetFromPeerRouteInfo(nFromPeerType, sFromPeerName, sDialNo, ref sDialToNo, ref sToPeerName, ref bRejected, ref sRoomName);

            if (bRejected)
            {
                cVaxServerSIP.mVaxSIP.RejectCallSession(nSessionId, 404, "Not Found");
                return;
            }
                                                
            OnProcessCall(nSessionId, sCallerName, sCallerId, sDialToNo, nFromPeerType, sFromPeerName, sUserAgentName, sFromIP, nFromPort, sToPeerName, sRoomName);
        }

        ///////////////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////////////

        #region Events

        private void OnProcessCall(int nSessionId, String sCallerName, String sCallerId, String sDialNo, int nFromPeerType, String sFromPeerName, String sUserAgentName, String sFromIP, int nFromPort, String sToPeerName, String sRoomName)
        {
            Boolean bProcessed = false;

            m_objVaxRoom.OnIncomingCall(nSessionId, sRoomName, ref bProcessed);
            if (bProcessed) return;
                        
            m_objVaxCall.OnIncomingCall(nSessionId, sCallerName, sCallerId, sDialNo, nFromPeerType, sFromPeerName, sToPeerName, ref bProcessed); // Process as Normal call.
        }
        
        ////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////

        public void OnCallSessionTransferBlind(int nTransfererSessionId, int nTransfererChannelId, String sTransferer, String sTransferee, String sTransferTo)
        {
            m_objVaxTransfer.OnCallSessionTransferBlind(nTransfererSessionId, nTransfererChannelId, sTransferer, sTransferee, sTransferTo);
        }

        public void OnCallSessionTransferConsult(int nTransfererSessionId, int nTransfererChannelId, int nTransferToSessionId, int nTransferToChannelId, String sTransferer, String sTransferee, String sTransferTo)
        {
            m_objVaxTransfer.OnCallSessionTransferConsult(nTransfererSessionId,  nTransfererChannelId,  nTransferToSessionId, nTransferToChannelId, sTransferer, sTransferee, sTransferTo);
        }
                                        
        ////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////

        public void OnDetectedDigitDTMF(int nSessionId, int nChannelId, String sDigitDTMF)
        {
            m_objVaxDTMF.OnDetectedDigitDTMF(nSessionId, nChannelId, sDigitDTMF);
        }

        ////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////

        public void OnCallSessionDetectedAMD(int nSessionId, int nChannelId, Boolean bIsHuman)
        {
            m_objVaxAMD.OnCallSessionDetectedAMD(nSessionId, nChannelId, bIsHuman);
        }

        ////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////
        public void OnCallSessionAccessAudioPCM(int nSessionId, int nAccessType, Byte[] aDataPCM, int nSizePCM)
        {
            m_objAccessPCM.OnCallSessionAccessAudioPCM(nSessionId, nAccessType, aDataPCM, nSizePCM);
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////
        public void OnCallSessionConnected(int nSessionId)
        {
            m_objAccessPCM.OnCallSessionConnected(nSessionId);
            m_objDetectVAD.OnCallSessionConnected(nSessionId);
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        public void OnCallSessionMediaDetectStartedVAD(int nSessionId, int nChannelId)
        {
            m_objDetectVAD.OnCallSessionMediaDetectStartedVAD(nSessionId, nChannelId);
        }

        public void OnCallSessionMediaDetectCompletedVAD(int nSessionId, int nChannelId, Byte[] aDataPCM, int nSizePCM, int nTypePCM)
        {
            m_objDetectVAD.OnCallSessionMediaDetectCompletedVAD(nSessionId, nChannelId, aDataPCM, nSizePCM, nTypePCM);
        }

        public void OnCallSessionMediaDetectExpiredVAD(int nSessionId, int nChannelId)
        {
            m_objDetectVAD.OnCallSessionMediaDetectExpiredVAD(nSessionId, nChannelId);
        }

        #endregion
    }
}
