﻿using System;
using System.Collections;
using System.Collections.Generic;

namespace ServerSIP
{
    class cUserRouteDetails : cVaxHashTextData
    {
        public String m_sUserPrefix = "", m_sToPeerName = "", m_sDialPrefix = "";
	    
	    public cUserRouteDetails() {}
    }

    class cLineRouteDetails : cVaxHashTextData
    {
        public String m_sLineName = "", m_sRoomName = "", m_sToPeerName = "";
        public Boolean m_bRejected = false;

	    public cLineRouteDetails(){}
    }
    
    ////////////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////

    class cVaxCallSessionRoute
    {
        cVaxHashText m_objListUserRoute = null;
        cVaxHashText m_objListLineRoute = null;
                
        public cVaxCallSessionRoute()
        {
            m_objListUserRoute = new cVaxHashText(64);
            m_objListLineRoute = new cVaxHashText(64);
        }
        	    
	    private cUserRouteDetails FindUserRoute(String sUserName)
        {
            return (cUserRouteDetails) m_objListUserRoute.FindData(sUserName);
        }
	    
        private cLineRouteDetails FindLineRoute(String sLineName)
        {
            return (cLineRouteDetails) m_objListLineRoute.FindData(sLineName);
        }

        //////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////

        private Boolean GetUserRouteDetails(String sUserName, ref String sUserPrefix, ref String sDialPrefix, ref String sToPeerName)
        {
            cUserRouteDetails objUserRoute = FindUserRoute(sUserName);
            if(objUserRoute == null) return false;

            sUserPrefix = objUserRoute.m_sUserPrefix;
            sDialPrefix = objUserRoute.m_sDialPrefix;
                        
            sToPeerName = objUserRoute.m_sToPeerName;
            return true;
        }

        private Boolean GetLineRouteDetails(String sLineName, ref String sToPeerName, ref Boolean bRejected, ref String sRoomName)
        {
            cLineRouteDetails objLineRoute = FindLineRoute(sLineName);
            if(objLineRoute == null) return false;
                    
            sToPeerName = objLineRoute.m_sToPeerName;
            bRejected = objLineRoute.m_bRejected;
            sRoomName = objLineRoute.m_sRoomName;
                        
            return true;
        }

        //////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////
       
	    public void AddUserRoute(String sUserName, String sUserPrefix, String sDialPrefix, String sToPeerName)
        {
            RemoveUserRoute(sUserName);

            cUserRouteDetails objUserRoute = new cUserRouteDetails();

            objUserRoute.m_sUserPrefix = sUserPrefix;
            objUserRoute.m_sDialPrefix = sDialPrefix;
            
            objUserRoute.m_sToPeerName = sToPeerName;
            m_objListUserRoute.AddData(sUserName, objUserRoute);
        }
	
        public void RemoveUserRoute(String sUserName)
        {
           m_objListUserRoute.RemoveData(sUserName);
        }

        //////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////

	    public void AddLineRoute(String sLineName, String sToPeerName, Boolean bRejected, String sRoomName)
        {
            RemoveLineRoute(sLineName);

            cLineRouteDetails objLineRoute = new cLineRouteDetails();
                    
            objLineRoute.m_sToPeerName = sToPeerName;
            objLineRoute.m_bRejected = bRejected;
            objLineRoute.m_sRoomName = sRoomName;
                        
            m_objListLineRoute.AddData(sLineName, objLineRoute);
        }
	
        public void RemoveLineRoute(String sLineName)
        {
            m_objListLineRoute.RemoveData(sLineName);
        }

        //////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////

        public Boolean GetUserRouteInfo(String sUserName, String sDialNo, ref String sDialToNo, ref String sToPeerName)
        {
            String sUserPrefix = "", sDialPrefix = "";

            Boolean bResult = GetUserRouteDetails(sUserName, ref sUserPrefix, ref sDialPrefix, ref sToPeerName);

            // USER-ROUTE not found. Make it U2U call.

            if (bResult == false || sToPeerName.Length == 0 || sUserPrefix.Length == 0)
            {
                sDialToNo = sDialNo;
                sToPeerName = sDialNo;

                return true;
            }

            int nUserPrefixLen = sUserPrefix.Length;

            // Check if first 'N' charachters are equal to sUserPrefix. If YES; Modify it according to dialplan.

            if (sDialNo.Substring(0, nUserPrefixLen) == sUserPrefix)
            {
                String sCallTo = sDialNo.Remove(0, nUserPrefixLen);
                sDialToNo = sDialPrefix + sCallTo;
            }
            else
            {
                //Make it USER_TO_USER call.

                sDialToNo = sDialNo;
                sToPeerName = sDialNo;
            }

            return true;
        }

        //////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////

        public void GetLineRouteInfo(String sLineName, String sDialNo, ref String sDialToNo, ref String sToPeerName, ref Boolean bRejected, ref String sRoomName)
        {
            Boolean bResult = GetLineRouteDetails(sLineName, ref sToPeerName, ref bRejected, ref sRoomName);

            if (bResult == true)
            {
                if(sToPeerName.Length != 0)
                    sDialToNo = sToPeerName;

                return;
            }

            sDialToNo = sDialNo;
            sToPeerName = sDialNo;
        }

        //////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////
    }
}
