﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using System.Text;

namespace ServerSIP
{
    class cVaxUser : cVaxHashTextData
    {
        private static cVaxUserList m_objUserList = null;
        private static int m_nRegisterUserCount = 0;
                
        private Boolean m_bBlocked = false, m_bRegistered = false;
        private int m_nVoiceTypeId = cVaxServerCOM.VAX_AUDIO_TYPE_NORMAL;

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        public cVaxUser(String sUserName, Boolean bBlocked, int nVoiceTypeId)
        {
            if (cVaxUser.m_objUserList == null)
                cVaxUser.m_objUserList = new cVaxUserList();
           
            m_bBlocked = bBlocked;
            m_nVoiceTypeId = nVoiceTypeId;
        }

        public static void ResetAll()
        {
            m_nRegisterUserCount = 0;
            cVaxUser.m_objUserList = new cVaxUserList();
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private static cVaxUser GetUser(String sUserName)
        {
            return m_objUserList.GetUser(sUserName);
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        public static void OnRegisterUser(String sUserName, String sDomain, String sUserAgentName, String sFromIP, int nFromPort, int nRegId)
        {
            cVaxUser objUser = cVaxUser.GetUser(sUserName);

            if (objUser == null)
            {
                cVaxServerSIP.mVaxSIP.RejectRegister(nRegId, 404, "Not Found");
                
                String sStatus = sUserName + " " + cVaxDefine.USER_NOT_FOUND;
                cVaxServerSIP.mVaxSIP.GenStatusLog(sStatus);
                
                return;
            }

            objUser.OnRegister(sUserName, nRegId);
        }

        private void OnRegister(String sUserName, int nRegId)
        {
            if (m_bBlocked)
            {
                cVaxServerSIP.mVaxSIP.RejectRegister(nRegId, 404, "Not Found");
                return;
            }

            cVaxServerSIP.mVaxSIP.AuthRegister(nRegId);

            String sStatus = sUserName + " " + cVaxDefine.USER_REG_TRYING;
            cVaxServerSIP.mVaxSIP.GenStatusLog(sStatus);
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        public static void OnRegisterUserSuccess(String sUserName, String sFromIP, int nFromPort, int nRegId)
        {
            cVaxUser objUser = cVaxUser.GetUser(sUserName);
            if (objUser == null) return;
            
            objUser.OnRegisterSuccess(sUserName);
        }

        private void OnRegisterSuccess(String sUserName)
        {
            if (m_bRegistered == false)
                m_nRegisterUserCount++;

            m_bRegistered = true;

	        String sStatus = sUserName + " " + cVaxDefine.USER_REG_SUCCESS;
            cVaxServerSIP.mVaxSIP.GenStatusLog(sStatus);
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        public static void OnRegisterUserFailed(String sUserName, String sFromIP, int nFromPort, int nRegId)
        {
            cVaxUser objUser = cVaxUser.GetUser(sUserName);
            if (objUser == null) return;

            objUser.OnRegisterFailed(sUserName);
        }

        private void OnRegisterFailed(String sUserName)
        {
            String sStatus = sUserName + " " + cVaxDefine.USER_REG_FAILED;
            cVaxServerSIP.mVaxSIP.GenStatusLog(sStatus);
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        public static void OnUnRegisterUser(String sUserName)
        {
            cVaxUser objUser = cVaxUser.GetUser(sUserName);
            if (objUser == null) return;

            objUser.OnUnRegister(sUserName);
        }

        private void OnUnRegister(String sUserName)
        {
            if (m_bRegistered == true)
                m_nRegisterUserCount--;
            
            m_bRegistered = false;

            String sStatus = sUserName + " " + cVaxDefine.USER_UNREGISTER;
            cVaxServerSIP.mVaxSIP.GenStatusLog(sStatus);
        }
                       
        //////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////
                     
        public static Boolean AddUser(String sUserName, String sPassword, String sAudioCodecList, Boolean bBlocked, int nVoiceTypeId, String sUserPrefix, String sDialPrefix, String sToPeerName)
        {
            if (cVaxServerSIP.mVaxSIP.IsInitialized() == false || (sUserName.Length == 0))
                return false;

            RemoveUser(sUserName);

            if (!cVaxServerSIP.mVaxSIP.AddUser(sUserName, sPassword, sAudioCodecList))
            {
                return false;
            }

            cVaxCallSession.AddUserRoute(sUserName, sUserPrefix, sDialPrefix, sToPeerName);
            m_objUserList.AddUser(sUserName, bBlocked, nVoiceTypeId);

            cStoreUser.AddUser(sUserName, sPassword, sAudioCodecList, bBlocked, nVoiceTypeId, sUserPrefix, sDialPrefix, sToPeerName);
            return true;
        }

        public static void RemoveUser(String sUserName)
        {
            if (cVaxServerSIP.mVaxSIP.IsInitialized() == false || (sUserName.Length == 0))
                return;

            cVaxServerSIP.mVaxSIP.RemoveUser(sUserName);

            cVaxCallSession.RemoveUserRoute(sUserName);
            m_objUserList.RemoveUser(sUserName);

            cStoreUser.RemoveUser(sUserName);
        }

        public static void LoadUser()
        {
            if (!cVaxServerSIP.mVaxSIP.IsInitialized())
                return;

            ArrayList objListUsers = new ArrayList();
            cStoreUser.GetUserAll(ref objListUsers);

            if (objListUsers.Count == 0)
                return;

            String sPassword = "", sAudioCodecList = "";
            String sUserPrefix = "", sDialPrefix = "", sToPeerName = "";

            Boolean bBlocked = false;
            int nVoiceTypeId = cVaxServerCOM.VAX_AUDIO_TYPE_NORMAL;

            foreach (String sUserName in objListUsers)
            {
                cStoreUser.GetUser(sUserName, ref sPassword, ref sAudioCodecList, ref bBlocked, ref nVoiceTypeId, ref sUserPrefix, ref sDialPrefix, ref sToPeerName);
                AddUser(sUserName, sPassword, sAudioCodecList, bBlocked, nVoiceTypeId, sUserPrefix, sDialPrefix, sToPeerName);
            }
        }

        public static Boolean IsUserTypeAdmin(String sUserName)
        {
            cVaxUser objUser = m_objUserList.GetUser(sUserName);
            if (objUser == null) return false;

            return (objUser.m_nVoiceTypeId == cVaxServerCOM.VAX_AUDIO_TYPE_ADMIN);
        }

        public static int GetVoiceTypeId(String sUserName)
        {
            cVaxUser objUser = m_objUserList.GetUser(sUserName);
            if (objUser == null) return cVaxServerCOM.VAX_AUDIO_TYPE_NORMAL;

            return objUser.m_nVoiceTypeId;
        }

        public static int GetRegisteredUserCount()
        {
            return m_nRegisterUserCount;
        }
    }
}
