﻿using System;
using System.Collections;

namespace ServerSIP
{
    class cStoreLine
    {
        const String LINE_ROOT_PATH = "Lines\\";

        //////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////

        public static Boolean AddLine(String sLineName, int nLineType, String sAuthUser, String sAuthPwd, String sServerAddr, int nServerPort, String sAudioCodecList, Boolean bRegister, String sToPeerName, Boolean bRejected, String sRoomName)
        {
            if (sLineName.Length == 0)
                return false;

            String sKeyPath = LINE_ROOT_PATH + sLineName;
            cStoreRegistry objStore = new cStoreRegistry();

            objStore.SetTEXT(sKeyPath, "LineName", sLineName);
            objStore.SetINT(sKeyPath, "LineType", nLineType);

            objStore.SetTEXT(sKeyPath, "AuthUser", sAuthUser);
            objStore.SetTEXT(sKeyPath, "AuthPwd", sAuthPwd);

            objStore.SetTEXT(sKeyPath, "ServerAddr", sServerAddr);
            objStore.SetINT(sKeyPath, "ServerPort", nServerPort);

            objStore.SetTEXT(sKeyPath, "AudioCodecList", sAudioCodecList);
            objStore.SetBOOL(sKeyPath, "Register", bRegister);

            objStore.Close();

            cStoreLine.AddLineRoute(sLineName, sToPeerName, bRejected, sRoomName);
            return true;
        }

        public static void RemoveLine(String sLineName)
        {
            if (sLineName.Length == 0)
                return;

            String sKeyPath = LINE_ROOT_PATH + sLineName;
            cStoreRegistry objStore = new cStoreRegistry();

            objStore.DeleteSubKey(sKeyPath);
            objStore.Close();

            cStoreLine.RemoveLineRoute(sLineName);
        }

      //////////////////////////////////////////////////////////////////////////////
      //////////////////////////////////////////////////////////////////////////////

        public static Boolean GetLine(String sLineName, ref int nLineType, ref String sAuthUser, ref String sAuthPwd, ref String sServerAddr, ref int nServerPort, ref String sAudioCodecList, ref Boolean bRegister, ref String sToPeerName, ref Boolean bRejected, ref String sRoomName)
        {
            if (sLineName.Length == 0)
                return false;

            String sKeyPath = LINE_ROOT_PATH + sLineName;
            cStoreRegistry objStore = new cStoreRegistry();

            objStore.GetINT(sKeyPath, "LineType", ref nLineType, cVaxServerCOM.VAX_LINE_TYPE_UDP);

            objStore.GetTEXT(sKeyPath, "AuthUser", ref sAuthUser, "");
            objStore.GetTEXT(sKeyPath, "AuthPwd", ref sAuthPwd, "");

            objStore.GetTEXT(sKeyPath, "ServerAddr", ref sServerAddr, "");
            objStore.GetINT(sKeyPath, "ServerPort", ref nServerPort, cVaxServerCOM.SIP_DEFAULT_PORT);

            objStore.GetTEXT(sKeyPath, "AudioCodecList", ref sAudioCodecList, "");
            objStore.GetBOOL(sKeyPath, "Register", ref bRegister, false);

            objStore.Close();

            cStoreLine.GetLineRoute(sLineName, ref sToPeerName, ref bRejected, ref sRoomName);
            return true;
        }

        public static void GetLineNameAll(ref ArrayList objList)
        {
            objList.Clear();

            String sKeyPath = LINE_ROOT_PATH;
            cStoreRegistry objStore = new cStoreRegistry();

            objStore.EnumSubKeys(sKeyPath, ref objList);
            objStore.Close();
        }

        /////////////////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////////

        private static Boolean AddLineRoute(String sLineName, String sToPeerName, Boolean bRejected, String sRoomName)
        {
            if(sLineName.Length == 0)
                return false;

            String sKeyPath = LINE_ROOT_PATH + sLineName;
            cStoreRegistry objStore = new cStoreRegistry();
                        
            objStore.SetTEXT(sKeyPath, "ToPeerName", sToPeerName);
            objStore.SetBOOL(sKeyPath, "Rejected", bRejected);
            objStore.SetTEXT(sKeyPath, "RoomName", sRoomName);
            
            objStore.Close();
            return true;
        }

        private static Boolean GetLineRoute(String sLineName, ref String sToPeerName, ref Boolean bRejected, ref String sRoomName)
        {
            if(sLineName.Length == 0)
                return false;

            String sKeyPath = LINE_ROOT_PATH + sLineName;
            cStoreRegistry objStore = new cStoreRegistry();
                        
            objStore.GetTEXT(sKeyPath, "ToPeerName", ref sToPeerName, "");
            objStore.GetBOOL(sKeyPath, "Rejected", ref bRejected, false);
            objStore.GetTEXT(sKeyPath, "RoomName", ref sRoomName, "");
            
            objStore.Close();
            return true;
        }

        private static void RemoveLineRoute(String sLineName)
        {
            if(sLineName.Length == 0)
                return;

            String sKeyPath = LINE_ROOT_PATH + sLineName;
            cStoreRegistry objStore = new cStoreRegistry();

            objStore.DeleteValue(sKeyPath, "ToPeerName");
            objStore.DeleteValue(sKeyPath, "Rejected");
            objStore.DeleteValue(sKeyPath, "RoomName");
            
            objStore.Close();
        }

        public static Boolean IsDataStored()
        {
            cStoreRegistry objStore = new cStoreRegistry();
            Boolean bResult = objStore.IsKeyExist(LINE_ROOT_PATH);

            objStore.Close();
            return bResult;
        }
    }
}
